#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <stdbool.h>
#include "rssi_list.h"

extern Element *rssi_list;

u_char *string_to_mac(char * buf, u_char * byte_mac){
    sscanf(buf,"%hhx:%hhx:%hhx:%hhx:%hhx:%hhx", &byte_mac[0], &byte_mac[1],&byte_mac[2],&byte_mac[3],&byte_mac[4],&byte_mac[5]);
	return byte_mac;
}

char * mac_to_string(u_char * byte_mac, char * buf){
    sprintf(buf,"%02hhx:%02hhx:%02hhx:%02hhx:%02hhx:%02hhx",byte_mac[0],
        byte_mac[1], byte_mac[2], byte_mac[3], byte_mac[4],byte_mac[5]);
        return buf;
}

unsigned long long get_time(){

    struct timeval time;
    gettimeofday(&time, NULL);
    return (unsigned long long)(time.tv_sec) * 1000 + (unsigned long long)(time.tv_usec) / 1000;
}

void clear_outdated_values(Deque * list){
		Rssi_sample * current = list->head;
		Rssi_sample * previous = NULL;
		while(current != NULL) {
			if(current->deadline <  get_time()) {
				if(previous == NULL) {
                    previous = current->next;
                    free(current);
                    if(previous!=NULL)
					{current = previous->next;}
				} else {
					previous->next = current->next;
					free(current);
					current = previous->next;
				}
			} else {
				previous = current;
				current = current->next;
			}
		}
}

void clear_values(Deque * list){
    Rssi_sample *current = list->head;
    Rssi_sample *next = NULL;
    while(current != NULL){
        next = current->next;
        free(current);
        current = next;
    }
    list->head = NULL;
    list->tail = NULL;
}

void add_value(Deque * list, int value){
    unsigned long long deadline = get_time() + DEFAULT_KEEP_DELAY;
    Rssi_sample *new_rssi = (Rssi_sample*)malloc(sizeof(Rssi_sample));
    new_rssi->deadline = deadline;
    new_rssi->rssi_mW = pow(10,(value/10));
    if(list->tail!=NULL){
        Rssi_sample *tmp = list->tail;
        tmp->next = new_rssi;
        list->tail = new_rssi;
    }else{
        list->head = list->tail = new_rssi;
    }
}

//we must clear rssi measurements
void clear_list(Element ** list){
	Element *tmp;
	tmp = *list;
	Deque *mesure;
	while (tmp!= NULL) {
		*list = tmp->next;
		mesure =tmp->measurements;
		clear_values(mesure);
		free(tmp);
		tmp = *list;
	}
}

Element * find_mac(Element * list, u_char * mac_value){
    Element *current = list;
    bool same =true;
    int i;
    while(current != NULL){
        for(i= 0; i < 6; i++){
            if(mac_value[i] != current->mac_addr[i])
                same = false;
        }
        if(same){
            return current;
        }
        current= current->next;
        same= true;
    }
    return current;
}

Element * add_element(Element ** list, u_char * mac_value){
    Element *current = *list;
    int i = 0;
    Element *new_element = (Element*)malloc(sizeof(Element));
    new_element->next = NULL;
    new_element->measurements = malloc(sizeof(Deque));
    new_element->measurements->head = NULL;
    new_element->measurements->tail = NULL;
    while(i < 6){
        new_element->mac_addr[i] = mac_value[i];
	i++;
    }
    if(current == NULL){
        current = new_element;
    }
    else{
    while(current->next != NULL)
        current = current->next;
        current->next = new_element;
    }
    return *list;
}

void delete_element(Element ** list, Element * e){
    Element *current = *list;
    Element *tmp = NULL;
    while( current!=NULL && current != e){
            tmp = current;
        current = current->next;
    }
    if(current!=NULL){
        tmp->next = current->next;
        clear_values(current->measurements);
        free(current);
    }
}

void clear_empty_macs(Element ** list){
	Element *tmp;
	tmp = *list;
	while (tmp!= NULL) {
        if(tmp->measurements->head == NULL){
		*list = tmp->next;
		free(tmp);
		tmp = *list;
            }
	}
}


float computeMean(Element *l, char mac_addr[6]) {
	float rssi_avg = 0;
	int nb_samples = 0;
	char * buf="";
	Element *temp = (Element*)malloc(sizeof(Element));
	temp = l;
	while (temp != NULL) {
		if(strcmp(mac_to_string(temp->mac_addr,buf), mac_addr)) {
			nb_samples ++;
			rssi_avg += computeDequeMean(l->measurements);
		}
		temp = temp->next;
	}
	return (rssi_avg/nb_samples);
}

float computeDequeMean(Deque *l){
	double avg = 0;
	int nbSamples = 0;
	Deque *tmp = (Deque*)malloc(sizeof(Deque));
	tmp = l;
	while(tmp->head != NULL){
		nbSamples++;
		avg += tmp->head->rssi_mW;
		tmp->head = tmp->head->next;
}
	return (avg/nbSamples);
}
char * build_element(Element * e){
    int nbSamples = 0;
    float rssi_avg = 0;
	char *to_char="";
	char *tmp="";
	char *buf = mac_to_string(e->mac_addr,tmp);
	Rssi_sample *rssi = e->measurements->head;
	while(rssi!=NULL){
            nbSamples++;
        rssi= rssi->next;
	}
    rssi_avg = computeDequeMean(e->measurements);
	sprintf(to_char, "%f", rssi_avg);
	buf= strcat(buf, ":");
	buf= strcat(buf, to_char);
	buf= strcat(buf, "samples:");
        sprintf(to_char, "%i", nbSamples);
	buf= strcat(buf, to_char);
    	return buf;
}

char * build_buffer(Element * list, char * buffer, char * my_name,
		    u_char * macs_requested,
		    unsigned short nb_macs){
    u_char *current_mac = macs_requested;
    Element* find;
	buffer= "{ap:";
	buffer= strcat(buffer, my_name);
	buffer= strcat(buffer, ",rssi:[");
    while(nb_macs>0){
            find = find_mac(list,current_mac);
        buffer = strcat(buffer,build_element(find));
     nb_macs--;
    }
    buffer= strcat(buffer, "}]}");
return buffer;
}

char * build_buffer_full(Element * list, char * buffer, char * my_name){
    Element *current = list;
	buffer= "{ap:";
	buffer= strcat(buffer, my_name);
	buffer= strcat(buffer, ",rssi:[");
    while(current!=NULL){
        buffer = strcat(buffer,build_element(current));
        current = current->next;
    }
    buffer= strcat(buffer, "}]}");
return buffer;
}


