package com.lo53_mobile_localization.core.servlet;

import com.lo53_mobile_localization.core.entity.AccessPoint;
import com.lo53_mobile_localization.core.entity.Location;
import com.lo53_mobile_localization.core.entity.Rssi;
import com.lo53_mobile_localization.core.service.AccessPointService;
import com.lo53_mobile_localization.core.service.LocationService;
import com.lo53_mobile_localization.core.service.RSSIService;
import com.lo53_mobile_localization.core.servlet.util.FakeApMessages;
import com.lo53_mobile_localization.core.servlet.util.Param;

import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.nio.charset.StandardCharsets;

/**
 * Created by Thomas on 18/05/2017.
 */
@WebServlet(
        name = "CalibrationServlet",
        urlPatterns = {"/calibration"}
)
public class CalibrationServlet extends HttpServlet {

    /**
	 * 
	 */
	private static final long serialVersionUID = -8542162228217771943L;
	
	DatagramSocket socket = null;

    public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        response.setContentType(Param.CONTENT_TYPE);
        request.getRequestDispatcher(Param.PATH_CALIBRATION).forward(request, response);
    }


    public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {

        String apRequestIp = request.getRemoteAddr();
        System.out.println("Request from " + apRequestIp + " received");

        int x = -1;
        int y = -1;
        try{
            x = Integer.parseInt(request.getParameter("positionX"));
            y = Integer.parseInt(request.getParameter("positionY"));

            System.out.println("x = " + x + " y = " + y);
        }catch (NumberFormatException nfe){
            System.out.println("Wrong parameters");
        }

        if (x != -1 && y != -1){

            //String macAddr = getMacAddressFromArpTable(apRequestIp);
            String macAddr = "12:12:12:12:12:12";

            if (macAddr != null){
                System.out.println("MAC Address found : " + macAddr);
                try{
                    for (int i = 0 ; i < Param.AP_IPS.length ; i++){

                        InetAddress address = InetAddress.getByAddress(Param.AP_IPS[i]);
                        socket = new DatagramSocket(Param.AP_COMM_PORT);

                        String msg = buildRssiRequestMessage(Param.AP_IPS[i], Param.AP_COMM_PORT, macAddr);

                        DatagramPacket packet = new DatagramPacket(msg.getBytes(), msg.length(), address, Param.AP_COMM_PORT);
                        socket.send(packet);
                        System.out.println("Sending "+msg+" to AP number " + i);
                    }
                    socket.close();
                } catch (IOException iox){
                    System.out.println(iox+" [Calibration Servlet] IOException");
                }finally{
                    if (socket != null) socket.close();
                }
            }else{
                System.out.println("Unknown mac address");
            }
        }else{
            System.out.println("Wrong coordinates");
        }

        long startTime = System.currentTimeMillis(); //starting time
        try {
            socket = new DatagramSocket(Param.DATAG_LISTENING_PORT);
            
            while (false || (System.currentTimeMillis() - startTime) < 500) {

                byte[] receiveData = new byte[1024];
                DatagramPacket packet = new DatagramPacket(receiveData, receiveData.length);
                socket.receive(packet);
                String strReceivedData = new String(packet.getData());

                //msg = "ap:ap:ap:ap:ap:ap" , "xx:xx:xx:xx:xx:xx" , "val" , "nbSamples"
                //String[] data = strReceivedData.split(",");
                String [] data = FakeApMessages.inventOneMessage().split(",");

                String apMacAddress = data[0];
                String deviceMac = data[1];
                double value = Double.parseDouble(data[2]);
                int nbSamples = Integer.parseInt(data[3]);

                if(nbSamples > 4){

                    LocationService locationService = new LocationService();
                    AccessPointService apService = new AccessPointService();
                    RSSIService rssiService = new RSSIService();

                    Location location = locationService.getLocationFromCoordinates(x,y);
                    if(location == null){

                        location.setX(x);
                        location.setY(y);
                        locationService.storeEntity(location);

                        System.out.println("LOCATION STORED ---");

                    }

                    AccessPoint ap = apService.getSingleAccessPointFromMacAdress(apMacAddress);
                    if(ap == null){

                        ap = new AccessPoint();
                        ap.setMac_address(apMacAddress);
                        ap.setLocation_id(location);

                        apService.storeEntity(ap);

                        System.out.println("AP STORED ---");

                    }

                    Rssi rssi = rssiService.getRssiWithLocationAndAP(location, ap);
                    if(rssi == null){

                        rssi.setLocation(location);
                        rssi.setAccessPoint(ap);
                        rssi.setOccurences(nbSamples);
                        rssi.setValue(value);

                        rssiService.storeEntity(rssi);

                        System.out.println("RSSI STORED ---");
                    }else{
                        rssi.setValue(value);
                        rssiService.updateEntity(rssi.getId(), rssi);
                    }
                }
            }
        }  catch (SocketException e) {
        System.out.println("[Calibration Receive answer] SocketException");
    } catch (IOException e) {
        System.out.println("[Calibration receive] IOException");
    } catch (Exception e) {
            System.out.println("Update from RSSI failed");
            e.printStackTrace();
        } finally{
        if (socket != null ) 	socket.close();
    }
        request.getRequestDispatcher(Param.PATH_HOME).forward(request, response);
    }

    private String getMacAddressFromArpTable(String ip) throws IOException {

        BufferedReader in = new BufferedReader(new FileReader("/proc/net/arp"));
        String strLine;
        String macAddr = null;

        while ((strLine = in.readLine()) != null) {
            System.out.println(strLine);
            if (strLine.contains(ip)) {
                String[] table = strLine.split(" ");
                for (int i = 0; i < table.length; i++) {
                    if (table[i].length() == 17)
                        macAddr = table[i];
                }
            }
        }
        return macAddr;
    }

    private String buildRssiRequestMessage(byte [] apIp, int port, String mac){

        String request = "http://";

        String ip = new String(apIp, StandardCharsets.UTF_8);

        String map_id = "1";
        String get = "/get";
        String macReq = "macs=";

        request += ip + ":" + port + get + "?";
        request += macReq + mac;

        System.out.println(request);

        return request;
    }

}
