package com.lo53_mobile_localization.core.servlet;

import com.lo53_mobile_localization.core.entity.AccessPoint;
import com.lo53_mobile_localization.core.entity.Location;
import com.lo53_mobile_localization.core.entity.Rssi;
import com.lo53_mobile_localization.core.service.AccessPointService;
import com.lo53_mobile_localization.core.service.LocationService;
import com.lo53_mobile_localization.core.service.RSSIService;
import com.lo53_mobile_localization.core.servlet.util.FakeApMessages;
import com.lo53_mobile_localization.core.servlet.util.Param;

import java.io.BufferedReader;

import java.io.FileReader;
import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.nio.charset.StandardCharsets;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


@WebServlet(
        name = "PositionServlet",
        urlPatterns = {"/position"}
)public class PositionServlet extends HttpServlet {

    /**
	 * 
	 */
	private static final long serialVersionUID = 6142878685562769291L;
	DatagramSocket socket = null;

    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

        String requestIp = request.getRemoteAddr();
        System.out.println("Request from ip : " + requestIp);

        //String macAddr = getMacAdressFromArpTable(requestIp);
        String macAddr = "12:12:12:12:12:12";

        if (macAddr != null) {
            try {
                for (int i = 0; i < Param.AP_IPS.length; i++) {
                    InetAddress address = InetAddress.getByAddress(Param.AP_IPS[i]);
                    socket = new DatagramSocket(Param.AP_COMM_PORT);

                    String msg = buildMessage(macAddr, Param.AP_IPS[i]);

                    DatagramPacket packet = new DatagramPacket(msg.getBytes(), msg.length(), address, Param.AP_COMM_PORT);
                    socket.send(packet);
                    System.out.println("Sending " + msg + " to AP number " + i);
                }
                socket.close();
            } catch (IOException iox) {
                System.out.println("Position Servlet : error when sending IOException");
            }

            try {
                socket = new DatagramSocket(Param.DATAG_LISTENING_PORT);

                long startTime = System.currentTimeMillis(); //fetch starting time
                //time out
                while (false || (System.currentTimeMillis() - startTime) < Param.WAIT_TIME_MS);

                    byte[] receiveData = new byte[1024];
                    DatagramPacket packet = new DatagramPacket(receiveData, receiveData.length);
                    socket.receive(packet);
                    String stReceivedData = new String(packet.getData());

                    //msg = "ap:ap:ap:ap:ap:ap" , "xx:xx:xx:xx:xx:xx" , "val" , "nbSamples"
                    String[] data = FakeApMessages.inventOneMessage().split(",");

                    String apMacAddress = data[0];
                    String deviceMac = data[1];
                    double value = Double.parseDouble(data[2]);
                    int nbSamples = Integer.parseInt(data[3]);

                    if(nbSamples > 4){

                        LocationService locationService = new LocationService();
                        AccessPointService apService = new AccessPointService();
                        RSSIService rssiService = new RSSIService();

                        AccessPoint ap = apService.getSingleAccessPointFromMacAdress(macAddr);
                        if(ap == null){
                            ap.setMac_address(macAddr);
                            apService.storeEntity(ap);

                            System.out.println("AP STORED --");
                        }
                        Rssi rssi = new Rssi();
                        rssi.setValue(value);
                        rssi.setOccurences(nbSamples);
                        rssi.setAccessPoint(ap);

                        Location loc = computeMinimumDistance(value);

                        rssi.setLocation(loc);
                        rssiService.storeEntity(rssi);

                        request.setAttribute("xLoc", loc.getX());
                        request.setAttribute("yLoc", loc.getY());

                        System.out.println("xLoc = " + loc.getX() + " yLoc = " + loc.getY());

                        request.getRequestDispatcher(Param.PATH_POSITION).forward(request,response);
                    }

            }  catch (SocketException e) {
                System.out.println("[PositionServlet] SocketException");
            } catch (IOException e) {
                System.out.println("[PositionServlet] IOException");
            }finally{
                if (socket != null ) 	socket.close();
            }
        }else{
            request.setAttribute("xLoc", 0);
            request.setAttribute("yLoc", 0);
        }

        request.getRequestDispatcher(Param.PATH_POSITION).forward(request,response);
    }


    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

        response.setContentType(Param.CONTENT_TYPE);
        request.getRequestDispatcher(Param.PATH_POSITION).forward(request,response);

    }


    private String getMacAdressFromArpTable(String ip) throws IOException {

        BufferedReader in = new BufferedReader(new FileReader("/proc/net/arp"));
        String strLine;
        String macAddr = null;

        while ((strLine = in.readLine()) != null) {
            System.out.println(strLine);
            if (strLine.contains(ip)) {
                String[] table = strLine.split(" ");
                //macAddr = table[24];
                for (int i = 0; i < table.length; i++) {
                    if (table[i].length() == 17)
                        macAddr = table[i];
                }
            }
        }
        return macAddr;
    }

    private String buildMessage(String mac, byte[] apIp){

        //String msg = http://<AP IP>:<PORT>/get?macs=XX:XX:XX:XX:XX

        String request = "http://";

        String ip = new String(apIp, StandardCharsets.UTF_8);
        int port = Param.AP_COMM_PORT;
        String map_id = "1";
        String positionServlet = "/position";

        request += ip + ":" + port + positionServlet + "?";
        request += "macs=" + mac;

        System.out.println(request);

        return request;
    }

    private Location computeMinimumDistance(double value){

        Location location = new Location();
        location.setX(200);
        location.setY(200);

        System.out.println("COMPUTING MINIMUM DISTANCE");

        return location;
    }

    private double rssi_distance( List<Rssi> rss1, List<Rssi> rss2) {
        double distance = 0.0;
        for (Rssi val:rss1) {
            for (Rssi val2 : rss2) {
                if (val.getAccessPoint().getMac_address() == val2.getAccessPoint().getMac_address()) {
                    if(!val.equals(rss2.get(rss2.size()-1)))
                        distance += (val.getValue() - val2.getValue()) * (val.getValue() - val2.getValue());
                    else
                        distance += (val.getValue() + 95.0) * (val.getValue() + 95.0);

                }
            }
        }
        for (Rssi val:rss2) {
            for (Rssi val2 : rss1) {
                if (val.getAccessPoint().getMac_address() == val2.getAccessPoint().getMac_address()) {
                    if(val.equals(rss1.get(rss1.size()-1)))
                        distance += (val.getValue() + 95.0) * (val.getValue() + 95.0);
                }
            }
        }
        return Math.sqrt(distance);
    }
    
    //TODO Implement with Rssis and not fingerprint
   /* Location closest_rssi( List<Fingerprint> fingerprints,  List<Rssi> mes_rssi) {
        double minimum_dist;
        Location minimum_dist_location = new Location();
        minimum_dist = Double.POSITIVE_INFINITY;
        Fingerprint fingerprint = new Fingerprint();
        for (Fingerprint finger:fingerprints) {
            if (minimum_dist > rssi_distance(finger.getRSSI(),mes_rssi)){
                minimum_dist = rssi_distance(finger.getRSSI(),mes_rssi);
                minimum_dist_location = fingerprint.getLocation();
            }
        }
        return minimum_dist_location;
    }*/

}