package com.lo53_mobile_localization.core.service;

import com.lo53_mobile_localization.core.entity.AccessPoint;
import com.lo53_mobile_localization.core.entity.IEntity;
import com.lo53_mobile_localization.core.entity.Location;
import com.lo53_mobile_localization.core.entity.Map;
import com.lo53_mobile_localization.core.helpers.HibernateTestHelper;

import com.lo53_mobile_localization.core.repository.AccessPointDAO;
import com.lo53_mobile_localization.core.repository.LocationDAO;
import com.lo53_mobile_localization.core.repository.MapDAO;
import org.junit.Test;

import java.util.ArrayList;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

/**
 * Created by Thomas on 10/05/2017.
 */
public abstract class BaseServiceTest extends HibernateTestHelper {

    protected IService service;
    protected IEntity item;

    @Test
    public void storeEntity() throws Exception {
        IEntity testingItem = storeMultipleTestingItems(1).get(0);

        assertTrue(service.getEntities().contains(testingItem));
    }

    protected abstract ArrayList<IEntity> storeMultipleTestingItems(int nbItems) throws Exception;

    @Test
    public void removeEntity() throws Exception {
        service.removeEntity(item.getId());
        assertEquals(0, service.getEntities().size());
    }


    @Test
    public void getEntity() throws Exception {
        IEntity storedItem = service.getEntity(item.getId());
        assertEquals(item, storedItem);
    }

    @Test
    public void getEntities() throws Exception {
        ArrayList<IEntity> addedItems = storeMultipleTestingItems(10);
        ArrayList<IEntity> storedItems = (ArrayList<IEntity>) service.getEntities();

        assertEquals(1 + addedItems.size(), storedItems.size());
        assertTrue(storedItems.contains(item));
        assertTrue(storedItems.containsAll(addedItems));
    }

    protected Map genMap(){

        MapDAO mapDAO = new MapDAO();

        Map map = new Map();
        map.setM_width(5);
        map.setM_height(5);
        map.setPx_height(10);
        map.setPx_width(10);
        map.setDescription("DescriptionTest" + 0);

        try {
            mapDAO.insertEntity(map);
        } catch (Exception ignored) {}
        return map;
    }

    protected Location genLocation() {
        LocationDAO locationDAO = new LocationDAO();

        Location location = new Location();
        location.setX(5);
        location.setY(8);

        try {
            locationDAO.insertEntity(location);
        } catch (Exception ignored) {}
        return location;
    }

    protected AccessPoint genAP(){

        AccessPointDAO apDAO = new AccessPointDAO();

        AccessPoint ap = new AccessPoint();
        ap.setLocation_id(genLocation());
        ap.setMac_address("BB:BB:BB:BB:BB:BB");

        try {
            apDAO.insertEntity(ap);
        }catch (Exception ignored){}

        return ap;
    }

}
