package com.lo53_mobile_localization.core.service;

import com.lo53_mobile_localization.core.entity.*;
import com.lo53_mobile_localization.core.helpers.HibernateTestHelper;
import org.junit.Before;
import org.junit.Test;

import java.util.ArrayList;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

/**
 * Created by Thomas on 10/05/2017.
 */
public class RssiServiceTest extends HibernateTestHelper{

    private Rssi item = null;
    private RSSIService service;

    @Before
    public void setUp() throws Exception {
        super.setUp();

        service = new RSSIService();
        Rssi rssi = new Rssi();

        Location loc = new Location();
        AccessPoint ap = new AccessPoint();

        LocationService locationService = new LocationService();
        AccessPointService apService = new AccessPointService();

        locationService.storeEntity(loc);
        apService.storeEntity(ap);

        rssi.setAccessPoint(ap);
        rssi.setLocation(loc);
        rssi.setValue(10);
        rssi.setOccurences(6);

        service.storeEntity(rssi);

        item = rssi;
    }

    protected ArrayList<Rssi> storeMultipleTestingItems(int nbItems) throws Exception {

        ArrayList<Rssi> rssis = new ArrayList<>();

        int startingId = 1;
        for(int i = startingId; i < nbItems + startingId; i++) {
            Rssi testingRssi = new Rssi();

            testingRssi.setOccurences(i);
            testingRssi.setValue(i);

            Location loc = new Location();
            AccessPoint ap = new AccessPoint();

            AccessPointService apService = new AccessPointService();
            LocationService locationService = new LocationService();

            locationService.storeEntity(loc);
            apService.storeEntity(ap);

            testingRssi.setAccessPoint(ap);
            testingRssi.setLocation(loc);

            service.storeEntity(testingRssi);

            rssis.add(testingRssi);
        }

        return rssis;
    }

    @Test
    public void updateEntity() throws Exception {
        Rssi rssi =  item;
        rssi.setValue(100);

        service.updateEntity(rssi.getId(), rssi);

        Rssi storedClient = service.getEntity(rssi.getId());

        assertEquals((int)rssi.getValue(), (int)storedClient.getValue());
    }

    @Test
    public void storeEntity() throws Exception {
        Rssi testingItem = storeMultipleTestingItems(1).get(0);

        assertTrue(service.getEntities().contains(testingItem));
    }


    @Test
    public void removeEntity() throws Exception {
        service.removeEntity(item.getId());
        assertEquals(0, service.getEntities().size());
    }


    @Test
    public void getEntity() throws Exception {
        Rssi storedItem = service.getEntity(item.getId());
        assertEquals(item, storedItem);
    }

    @Test
    public void getEntities() throws Exception {
        ArrayList<Rssi> addedItems = storeMultipleTestingItems(10);
        ArrayList<Rssi> storedItems = (ArrayList<Rssi>) service.getEntities();

        assertEquals(1 + addedItems.size(), storedItems.size());
        assertTrue(storedItems.contains(item));
        assertTrue(storedItems.containsAll(addedItems));
    }

    @Test
    public void getRssiWithLocationAndAP(){

        Rssi testRssi = service.getRssiWithLocationAndAP(item.getLocation(), item.getAccessPoint());

        assertEquals(item, testRssi);

    }


}
